
######################################################################################################
#									
# 	This R Program simulates single dose-finding trials using the
#	Bayesian optimal interval method* and the Keyboard method**
#
#	*Liu, S., Yuan, Y. (2015). Bayesian optimal interval designs for phase I clinical trials. Journal
#			of the Royal Statistical Society - Series C (Applied Statistics) 64, 507-523.
#
#	**Yan, F., Mandrekar, S.J., Yuan, Y. (2017). Keyboard: a novel Bayesian toxicity probability
#			interval design for phase I clinical trials. Clinical Cancer Research 23, 3994-4003.
#
#######################################################################################################

###install required R packages
#install.packages("Iso")
library(Iso)

###Load the function 'bayescrm' 
interval_examples<-function(method,dectable,truth,target,start){
N=dim(dectable)[2]
obs=data.frame(cbind(1:N,rep(0,N),rep(0,N),rep(0,N),rep(0,N),rep(0,N),rep(0,N)))
names(obs)<-c('Patient','Tolerance','Level','DLT','I_E','I_D','phat_curr')
obs$Tolerance=round(runif(N),3)
stop=mtd=0; 
curr=start
y=n=numeric(length(truth));

for(i in 1:N){
    	obs$Level[i]<-curr
    	currtox <- obs$Tolerance[i]<=truth[obs$Level[i]]
	curr<-obs$Level[i]
	obs$DLT[i]<-currtox
	for(j in 1:length(truth)){
		y[j]=sum(obs$DLT[obs$Level==j])
		n[j]=sum(obs$Level==j)
	}
	obs$phat_curr[i]<-round(y[curr]/n[curr],3)

		##BEGIN dose-finding algorithm
		if(y[curr]<=dectable[2,n[curr]]){
			rec<-ifelse(curr==length(truth),curr,curr+1)
		} else if(y[curr]>=dectable[3,n[curr]]){
			rec<-ifelse(curr==1,curr,curr-1)
		} else {
			rec<-curr
			}

		if(n[rec]>=3 && y[rec]>=dectable[4,n[rec]]){
			rec<-ifelse(rec==1,rec,rec-1)
		}
		
		if(rec==1 && n[rec]>=3 && y[rec]>=dectable[4,n[rec]]){
			stop=1
			break
			}	
				
		if(currtox==0 & rec<curr){obs$I_D[i]<-1}
		if(currtox==1 & rec>curr){obs$I_E[i]<-1}

		curr<-rec			
		##END dose-finding algorithm

		
	if(stop==0){
		trtd=(n!=0)
		pipost=pava((y[trtd] + 0.005)/(n[trtd] + 0.01), w = 1/((y[trtd] + 
            0.005) * (n[trtd] - y[trtd] + 0.005)/((n[trtd] + 0.01)^2 * 
            (n[trtd] + 0.01 + 1))))
		mtd<-order(abs(pipost-target))[1]
	}

}
 cat("Simulation example for the", method,"method targeting a DLT rate of", target,"\n\n");
 cat("Recommended MTD:\n");
 cat(mtd, sep="\t",  "\n");
 cat("Number of incoherent de-escalation assignments:\n");
 cat(sum(obs$I_D), sep="\t",  "\n");
 cat("Number of incoherent escalation assignments:\n");
 cat(sum(obs$I_E), sep="\t",  "\n\n");
 cat("Simulated trial:\n");
 print(obs)
}

r1<-c(0.26,0.34,0.47,0.64,0.66,0.77)
r2<-c(0.18,0.25,0.32,0.36,0.60,0.69)
r3<-c(0.09,0.16,0.23,0.34,0.51,0.74)


####################################
#
#	Input
#	
#	method = interval-based dose-finding method
#	dectable = decision table for method
#	truth = true DLT probabilities 
#	target = target DLT rate
#	start = starting dose
# 
####################################

##Set directory
dir<-getwd()
dectable<-as.matrix(read.csv(paste(dir,"\\BOIN25.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
dectable<-as.matrix(read.csv(paste(dir,"\\Keyboard25.csv",sep=""), header = TRUE, row.names = 1,sep = ","))

truth<-r3       
target=0.25    
start=1     
method="Keyboard"
interval_examples(method,dectable,truth,target,start)


