######################################################################################################
#									
# 	This R Program simulates coherence operating characteristics of the
#	Bayesian optimal interval method* and the Keyboard method**
#
#	*Liu, S., Yuan, Y. (2015). Bayesian optimal interval designs for phase I clinical trials. Journal
#			of the Royal Statistical Society - Series C (Applied Statistics) 64, 507-523.
#
#	**Yan, F., Mandrekar, S.J., Yuan, Y. (2017). Keyboard: a novel Bayesian toxicity probability
#			interval design for phase I clinical trials. Clinical Cancer Research 23, 3994-4003.
#
#######################################################################################################
#install required R packages
#install.packages("Iso")
library(Iso)

###Load the function 'modelassisted' 
modelassisted<-function(dectable,truth,target,start){
 
### run a trial 	
    ndose = length(truth);   #number of combos
    y=n=rep(0,ndose);  #number of toxicities and number of treated patients at each dose level
    curr = start;  # current dose level	 
    dose.select=rep(0,ndose); # a vector of indicators for dose selection
    stop=0; #indicate if trial stops early
    ie=id=0
    i=1	

while(i <= dim(dectable)[2])
    {
	# generate data for a new cohort of patients
		currtox = rbinom(1,1,truth[curr])
		y[curr] = y[curr] + currtox;
		n[curr] = n[curr] + 1;
			
		if(y[curr]<=dectable[2,n[curr]]){
			rec<-ifelse(curr==ndose,curr,curr+1)
		} else if(y[curr]>=dectable[3,n[curr]]){
			rec<-ifelse(curr==1,curr,curr-1)
		} else {
			rec<-curr
			}

		if(n[rec]>=3 && y[rec]>=dectable[4,n[rec]]){
			rec<-ifelse(rec==1,rec,rec-1)
		}
		
		if(rec==1 && n[rec]>=3 && y[rec]>=dectable[4,n[rec]]){
			stop=1
			break
			}	
				
		if(currtox==0 & rec<curr){id=id+1}
		if(currtox==1 & rec>curr){ie=ie+1}
		
		curr<-rec				

		i=i+1
		
   	}
	if(stop==0){
		trtd=(n!=0)
		pipost=pava((y[trtd] + 0.005)/(n[trtd] + 0.01), w = 1/((y[trtd] + 
            0.005) * (n[trtd] - y[trtd] + 0.005)/((n[trtd] + 0.01)^2 * 
            (n[trtd] + 0.01 + 1))))
		mtd<-order(abs(pipost-target))[1]
		dose.select[mtd]=dose.select[mtd]+1;
		}
	return(list(dose.select=dose.select,tox.data=y,pt.allocation=n,stop=stop,ie=ie,id=id))
}
##########'modelassisted' end here

###Load the function 'bpocrm.sim' 
modelassisted.sim<-function(dectable,truth,target,start,ntrial){
	ndose=length(truth)
	
	dose.select<-y<-n<-matrix(nrow=ntrial,ncol=ndose)
	incod=incoe=rep(0,ntrial)
	nstop=0
	
	for(i in 1:ntrial){
		result<-modelassisted(dectable,truth,target,start)
		dose.select[i,]=result$dose.select
		y[i,]=result$tox.data
		n[i,]=result$pt.allocation
		incod[i]=result$id
		incoe[i]=result$ie
		nstop=nstop+result$stop
	}

	output<-list(true.prob=round(truth,3),MTDselection=round(colMeans(dose.select)*100,1),
				patient.allocation=round(colMeans(n),1),
				Number.DLT=round(colMeans(y),1),incoD=incod,incoE=incoe,stopped=nstop/ntrial*100,percinc=round(sum(incod>0|incoe>0)/ntrial*100,2))
				#,medD=round(median(incod),2),medE=round(median(incod),2),icoherD=round(max(incod),2),icoherE=round(max(incoe),2)
    }
##########'modelassisted.sim' end here



##Set directory
dir<-getwd()

####################################
#
#	Target DLT rate 0.30
# 
####################################
target=0.30    ##target toxicity rate
start=1        ##starting dose level
ntrial=1000    ##number of simulated trials under each curve
data<-data.matrix(read.csv(paste(dir,"\\sim7674-sup-0003-target30scenarios.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
dimnames(data) <- NULL

set.seed(580)
par(mfrow=c(1,2))

numcurves=100
curves30=matrix(,ncol=dim(data)[2],nrow=numcurves)
samp=sample(1:dim(data)[1],nrow(curves30))
for(i in 1:length(samp)){
	curves30[i,]<-data[samp[i],]
	}
matplot(t(curves30),type="l",lty=1,col="black",ylim=c(0,1),xlab="Dose level",ylab="Probability of DLT",cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=3,text="Target DLT rate = 30%" ,line=1,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)


###BOIN
dectable<-as.matrix(read.csv(paste(dir,"\\BOIN30.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
correctsel=desci=esci=iperc30=rep(0,length(samp))
imatD30=imatE30=matrix(,nrow=ntrial,ncol=length(samp))
for(i in 1:length(samp)){
	truth<-as.vector(curves30[i,])
	foo<-modelassisted.sim(dectable,truth,target,start,ntrial)
	#desci[i]=foo$icoherD
	#esci[i]=foo$icoherE
	iperc30[i]=foo$percinc
	imatD30[,i]=foo$incoD
	imatE30[,i]=foo$incoE
}
boin30<-as.vector(imatD30+imatE30)
b30<-data.frame(Method=rep("BOIN",length(table(boin30)/(numcurves*ntrial))),as.data.frame(table(boin30)/(numcurves*ntrial)))
cpb30<-rev(cumsum(rev(b30$Freq)))

###Keyboard
dectable<-as.matrix(read.csv(paste(dir,"\\Keyboard30.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
correctsel=desci=esci=iperc30=rep(0,length(samp))
imatD30=imatE30=matrix(,nrow=ntrial,ncol=length(samp))
for(i in 1:length(samp)){
	truth<-as.vector(curves30[i,])
	foo<-modelassisted.sim(dectable,truth,target,start,ntrial)
	#desci[i]=foo$icoherD
	#esci[i]=foo$icoherE
	iperc30[i]=foo$percinc
	imatD30[,i]=foo$incoD
	imatE30[,i]=foo$incoE
}
key30<-as.vector(imatD30+imatE30)
k30<-data.frame(Method=rep("Keyboard",length(table(key30)/(numcurves*ntrial))),as.data.frame(table(key30)/(numcurves*ntrial)))
cpk30<-rev(cumsum(rev(k30$Freq)))

G1<-matrix(,nrow=2,ncol=length(cpb30))
G1[1,]<-cpk30
G1[2,]<-cpb30
rownames(G1)<-c("Keyboard","BOIN")
colnames(G1)<-c(expression("">=0),expression("">=1),expression("">=2),expression("">=3),expression("">=4),
			expression("">=5),expression("">=6),expression("">=7),expression("">=8),expression("">=9),
			expression("">=10),expression("">=11),expression("">=12))
barplot(G1, beside=T,names.arg=c(expression("">=0),expression("">=1),expression("">=2),expression("">=3),expression("">=4),
			expression("">=5),expression("">=6),expression("">=7),expression("">=8),expression("">=9),
			expression("">=10),expression("">=11),expression("">=12)),col=c("black","gray"),xlab="",ylim=c(0,1.05),cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=2,text="Proportion of simulated trials",line=2.5,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=1,text="Number of incoherent decisions",line=3,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=3,text="Target DLT rate = 30%" ,line=1,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
legend("topright", c("Keyboard","BOIN"), col=c("black","gray"),pch=15, bty="n", cex = 1.75)
box(lty = 'solid',lwd='1', col = 'black')

####################################
#
#	Target DLT rate 0.20
# 
####################################
target=0.20    ##target toxicity rate
start=1        ##starting dose
ntrial=1000    ##number of simulated trials 
data<-data.matrix(read.csv(paste(dir,"\\sim7674-sup-0002-target20scenarios.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
dimnames(data) <- NULL

set.seed(580)
par(mfrow=c(1,2))

numcurves=100
curves20=matrix(,ncol=dim(data)[2],nrow=numcurves)
samp=sample(1:dim(data)[1],nrow(curves20))
for(i in 1:length(samp)){
	curves20[i,]<-data[samp[i],]
	}
matplot(t(curves20),type="l",lty=1,col="black",ylim=c(0,1),xlab="Dose level",ylab="Probability of DLT",cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=3,text="Target DLT rate = 20%" ,line=1,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)


###BOIN
dectable<-as.matrix(read.csv(paste(dir,"\\BOIN20.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
correctsel=desci=esci=iperc20=rep(0,length(samp))
imatD20=imatE20=matrix(,nrow=ntrial,ncol=length(samp))
for(i in 1:length(samp)){
	truth<-as.vector(curves20[i,])
	foo<-modelassisted.sim(dectable,truth,target,start,ntrial)
	#desci[i]=foo$icoherD
	#esci[i]=foo$icoherE
	iperc20[i]=foo$percinc
	imatD20[,i]=foo$incoD
	imatE20[,i]=foo$incoE
}
boin20<-as.vector(imatD20+imatE20)
b20<-data.frame(Method=rep("BOIN",length(table(boin20)/(numcurves*ntrial))),as.data.frame(table(boin20)/(numcurves*ntrial)))
cpb20<-rev(cumsum(rev(b20$Freq)))

###Keyboard
dectable<-as.matrix(read.csv(paste(dir,"\\Keyboard20.csv",sep=""), header = TRUE, row.names = 1,sep = ","))
correctsel=desci=esci=iperc20=rep(0,length(samp))
imatD20=imatE20=matrix(,nrow=ntrial,ncol=length(samp))
for(i in 1:length(samp)){
	truth<-as.vector(curves20[i,])
	foo<-modelassisted.sim(dectable,truth,target,start,ntrial)
	#desci[i]=foo$icoherD
	#esci[i]=foo$icoherE
	iperc20[i]=foo$percinc
	imatD20[,i]=foo$incoD
	imatE20[,i]=foo$incoE
}
key20<-as.vector(imatD20+imatE20)
k20<-data.frame(Method=rep("Keyboard",length(table(key20)/(numcurves*ntrial))),as.data.frame(table(key20)/(numcurves*ntrial)))
cpk20<-rev(cumsum(rev(k20$Freq)))

G2<-matrix(,nrow=2,ncol=length(cpk20))
G2[1,]<-cpk20
G2[2,]<-cpb20
rownames(G2)<-c("Keyboard","BOIN")
colnames(G2)<-c(expression("">=0),expression("">=1),expression("">=2),expression("">=3),expression("">=4),
			expression("">=5),expression("">=6),expression("">=7),expression("">=8),expression("">=9),
			expression("">=10),expression("">=11),expression("">=12),expression("">=13))
barplot(G2, beside=T,names.arg=c(expression("">=0),expression("">=1),expression("">=2),expression("">=3),expression("">=4),
			expression("">=5),expression("">=6),expression("">=7),expression("">=8),expression("">=9),
			expression("">=10),expression("">=11),expression("">=12),expression("">=13)),col=c("black","gray"),xlab="",ylim=c(0,1.05),cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=2,text="Proportion of simulated trials",line=2.5,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=1,text="Number of incoherent decisions",line=3,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
mtext(side=3,text="Target DLT rate = 20%" ,line=1,cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)
legend("topright", c("Keyboard","BOIN"), col=c("black","gray"),pch=15, bty="n", cex = 1.75)
box(lty = 'solid',lwd='1', col = 'black')

